@@ -38,32 +38,10 @@ static inline void count_dire_plus(CPInfo *, CPDD *);
 static inline void ch_time_and_mod_dir(const CPInfo *, const struct stat *, CPDD *);
 static inline void clean(const char [], const int);
 static inline void fgets_is_null(void);
-static inline void set_same_or_different(const VList *, const char *, const char *, CPInfo *);
+static inline void set_same_or_different(const VList *, const char *, const char *, CPInfo *, OPArg *);
 
 /*******************************************************************************
 *******************************************************************************/
-#define DRIVE_SAME_OR_DIFFERENT \
-{\
-	switch(oparg->THREAD_MODE)\
-	{\
-	case DRIVE_AUTO:\
-		set_same_or_different(drive_list, tmp_from, tmp_to, cpinfo);\
-		break;\
-\
-	case DRIVE_DIFFERENT:\
-		cpinfo->copy_mode = DIFFERENT;\
-		break;\
-\
-	case DRIVE_SAME:\
-		cpinfo->copy_mode = SAME;\
-		break;\
-\
-	default:\
-		cpinfo->copy_mode = SAME;\
-		break;\
-	}\
-}
-
 #define LINK_COPY \
 {\
 	count_link_plus(cpinfo, cpdd);\
@@ -74,12 +52,8 @@ static inline void set_same_or_different(const VList *, const char *, const char
 #define FILE_COPY \
 {\
 	count_file_plus(cpinfo, cpdd);\
-	DRIVE_SAME_OR_DIFFERENT \
-\
-	{\
-		check_overwrite(oparg, sdir, cpinfo, cpdd);\
-	}\
-\
+	set_same_or_different(drive_list, tmp_from, tmp_to, cpinfo, oparg); \
+	check_overwrite(oparg, sdir, cpinfo, cpdd);\
 	file_list = append_list(file_list, (void *)cpinfo);\
 }
 
@@ -135,7 +109,7 @@ static inline void set_same_or_different(const VList *, const char *, const char
 \
 	if(cmp_flag == false)\
 	{\
-		DRIVE_SAME_OR_DIFFERENT \
+		set_same_or_different(drive_list, tmp_from, tmp_to, cpinfo, oparg); \
 \
 		struct stat *stat_buf;\
 \
@@ -1092,7 +1066,7 @@ for(;;)\
 /******************************************************************************
  * ドライブが同一かどうか調べる
 *******************************************************************************/
-static inline void set_same_or_different(const VList *drive_list, const char *from, const char *to, CPInfo *cpinfo)
+static inline void set_same_or_different(const VList *drive_list, const char *from, const char *to, CPInfo *cpinfo, OPArg *oparg)
 {
 	/*
 	 * キャストしないと警告が出る
@@ -1174,4 +1148,19 @@ DEV_CHECK_SKIP:
 	{
 		cpinfo->copy_mode = SAME;
 	}
+
+	switch(oparg->THREAD_MODE)
+	{
+	case DRIVE_SAME:
+		cpinfo->copy_mode = SAME;
+		break;
+	case DRIVE_DIFFERENT:
+		cpinfo->copy_mode = DIFFERENT;
+		break;
+	case DRIVE_AUTO:
+		break;
+	default:
+		cpinfo->copy_mode = SAME;
+		break;
+	}
 }
