@@ -30,7 +30,7 @@
 	}\
 }
 
-// 関数プロトタイプ
+/* 関数プロトタイプ */
 OPArg * read_conf(const SDir *sdir);
 
 /*******************************************************************************
@@ -52,7 +52,7 @@ OPArg * read_conf(const SDir *sdir)
 	oparg->WRITE_MODE = SIZE_OR_TIME;
 	oparg->THREAD_MODE = DRIVE_AUTO;
 
-	// そもそも~/.config/snowcpが存在するかが重要だ
+	/* そもそも~/.config/snowcpが存在するかが重要だ */
 	if(g_file_test(sdir->user_settings_dir, G_FILE_TEST_EXISTS) == TRUE)
 	{
 		if(chdir(sdir->user_settings_dir) == 0)
@@ -72,9 +72,9 @@ OPArg * read_conf(const SDir *sdir)
 				{
 					switch(buf[0])
 					{
-					// 設定ファイルのバージョン
+					/* 設定ファイルのバージョン */
 					case '!':
-						// 改行文字をヌル文字に
+						/* 改行文字をヌル文字に */
 						buf[strlen(buf) - 1] = '\0';
 						char *ver = strtok(buf, "," );
 						ver = strtok(NULL, "," );
@@ -94,20 +94,20 @@ OPArg * read_conf(const SDir *sdir)
 							exit(EXIT_FAILURE);
 						}
 						break;
-					// SHA1SUM
+					/* SHA1SUM */
 					case '1':
 						oparg->CHECK = ONE;
 						HASH_CHECK_MODE_SET
 						break;
-					// MD5SUM
+					/* MD5SUM */
 					case '5':
 						oparg->CHECK = FIVE;
 						HASH_CHECK_MODE_SET
 						break;
-					// バッファサイズを指定
+					/* バッファサイズを指定 */
 					case 'b':
 						{
-							// 改行文字をヌル文字に
+							/* 改行文字をヌル文字に */
 							buf[strlen(buf) - 1] = '\0';
 							char *cp = strtok(buf, "," );
 							cp = strtok(NULL, "," );
@@ -123,10 +123,10 @@ OPArg * read_conf(const SDir *sdir)
 									exit(EXIT_FAILURE);
 								}
 
-								// サイズをメガバイト単位にする
+								/* サイズをメガバイト単位にする */
 								oparg->buffer_size = 1024 * 1024 * b;
 
-								if(oparg->buffer_size <= 2)
+								if(oparg->buffer_size < 2)
 								{
 									fprintf(stderr, "バッファサイズが2以下です\n");
 									exit(EXIT_FAILURE);
@@ -139,30 +139,32 @@ OPArg * read_conf(const SDir *sdir)
 							}
 						}
 						break;
-					// コンペア
+					/* コンペア */
+/*
 					case 'c':
 						oparg->CHECK = MEMCMP;
 						HASH_CHECK_MODE_SET
 						break;
-					// 上書き時に確認する
+*/
+					/* 上書き時に確認する */
 					case 'i':
 						oparg->I = INTERACTIVE;
 						oparg->WRITE_MODE = OVERWRITE;
 						break;
-					// コンペアのログを保存する
+					/* コンペアのログを保存する */
 					case 'l':
 						oparg->L = LOG;
 						break;
-					// 移動モード
+					/* 移動モード */
 					case 'm':
 						oparg->M = MOVE;
 						break;
-					// ベリファイ（MD5）
+					/* ベリファイ（MD5） */
 					case 'r':
 						oparg->CHECK = V_FIVE;
 						HASH_CHECK_MODE_SET
 						break;
-					// スレッドモード
+					/* スレッドモード */
 					case 't':
 						switch(buf[2])
 						{
@@ -184,11 +186,11 @@ OPArg * read_conf(const SDir *sdir)
 							break;
 						}
 						break;
-					// 表示モード
+					/* 表示モード */
 					case 'v':
 						oparg->V = VERBOS;
 						break;
-					// 上書きモード
+					/* 上書きモード */
 					case 'w':
 						oparg->I = NOT;
 
