@@ -4,6 +4,8 @@
 */
 #define _GNU_SOURCE
 
+#define USE_O_DIRECT_SIZE 1024 * 1024 * 200
+
 #include "ch_time_and_mod.h" // inline
 #include "compare_hash.h"
 #include "print_error.h" // inline
@@ -30,28 +32,21 @@
 #define TO_OPEN \
 {\
 	errno = 0;\
-	if((ito = open(cpinfo->to, O_CREAT | O_WRONLY | O_TRUNC | O_NOFOLLOW | O_NOATIME, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)) == -1)\
+	if(o_direct_write_flag == false)\
 	{\
-		switch(errno)\
-		{\
-		case EPERM:\
-			errno = 0;\
+		ito = open(cpinfo->to, O_CREAT | O_WRONLY | O_TRUNC | O_NOFOLLOW, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);\
+		o_direct_write_open = false;\
+	}\
+	else \
+	{\
+		ito = open(cpinfo->to, O_CREAT | O_WRONLY | O_TRUNC | O_NOFOLLOW | O_DIRECT, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);\
+		o_direct_write_open = true;\
+	}\
 \
-			if((ito = open(cpinfo->to, O_CREAT | O_WRONLY | O_TRUNC | O_NOFOLLOW, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)) == -1)\
-			{\
-				print_error("open", __FILE__, __LINE__, cpdd);\
-			}\
-			else \
-			{\
-				to_open = true;\
-				cpdd->mk_file_count++;\
-			}\
-			break;\
-\
-		default:\
-			print_error("open", __FILE__, __LINE__, cpdd);\
-			break;\
-		}\
+	if(ito == -1)\
+	{\
+		o_direct_write_open = false;\
+		print_error("open", __FILE__, __LINE__, cpdd);\
 	}\
 	else \
 	{\
@@ -107,13 +102,15 @@ static int ito;
 static int buffer_size;
 static int buffer_size_half;
 static int buffer_size_thread;
-static int rbuf_num;
-static int wbuf_num;
+static int ri;
+static int wi;
 static long psize;
 static _Bool from_open;
 static _Bool from_stat;
 static _Bool to_open;
 static _Bool verify;
+static _Bool o_direct_write_flag;
+static _Bool o_direct_write_open;
 
 /* ※注意。グローバル変数。 */
 char *write_now;
@@ -121,8 +118,11 @@ char *write_now;
 /* 関数プロトタイプ */
 void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd);
 inline static void mw_small(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
+inline static void mw_small_direct(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
 inline static void mw_big(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
+inline static void mw_big_direct(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
 inline static void mw_thread(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
+inline static void mw_thread_direct(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
 static void * read_thread_func(void *unknown);
 static void * do_md5_append(void *tmp);
 
@@ -149,8 +149,8 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 	total_write_error = 0;
 	ifrom = 0;
 	ito = 0;
-	rbuf_num = 0;
-	wbuf_num = 0;
+	ri = 0;
+	wi = 0;
 
 	if(buffer_size == 0)
 	{
@@ -187,8 +187,6 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 			(
 				((tofs_tmp[0] == 'e') && (tofs_tmp[1] == 'x') && (tofs_tmp[2] == 't') && (tofs_tmp[3] == '4'))
 				||
-				((tofs_tmp[0] == 'x') && (tofs_tmp[1] == 'f') && (tofs_tmp[2] == 's'))
-				||
 				((tofs_tmp[0] == 'b') && (tofs_tmp[1] == 't') && (tofs_tmp[2] == 'r') && (tofs_tmp[2] == 'f') && (tofs_tmp[2] == 's'))
 				||
 				((tofs_tmp[0] == 'o') && (tofs_tmp[1] == 'c') && (tofs_tmp[2] == 'f') && (tofs_tmp[2] == 's') && (tofs_tmp[2] == '2'))
@@ -196,6 +194,11 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 			{
 				fallocate_support = true;
 			}
+			else if((tofs_tmp[0] == 'x') && (tofs_tmp[1] == 'f') && (tofs_tmp[2] == 's'))
+			{
+				fallocate_support = true;
+				o_direct_write_flag = true;
+			}
 			else if
 			(
 				((tofs_tmp[0] == 'n') && (tofs_tmp[1] == 't') && (tofs_tmp[2] == 'f') && (tofs_tmp[3] == 's'))
@@ -261,6 +264,11 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 			from_stat = true;
 			from_size_local = stat_buf.st_size;
 			cpdd->total_source += from_size_local;
+
+			if(from_size_local < USE_O_DIRECT_SIZE)
+			{
+				o_direct_write_flag = false;
+			}
 		}
 
 		if((cpinfo->file_test == false) || (oparg->WRITE_MODE == OVERWRITE))
@@ -363,15 +371,44 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 		/* 1048576 == 1024 * 1024 == 1MB */
 		if(1048576 > from_size_local)
 		{
-			mw_small(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			if(o_direct_write_open == false)
+			{
+				mw_small(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			}
+			else
+			{
+				mw_small_direct(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			}
 		}
 		else if(cpinfo->copy_mode == DIFFERENT)
 		{
-			mw_thread(oparg, cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			if(o_direct_write_open == false)
+			{
+				mw_thread(oparg, cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			}
+			else
+			{
+				mw_thread_direct(oparg, cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			}
 		}
 		else
 		{
-			mw_big(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			if(o_direct_write_open == false)
+			{
+				mw_big(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			}
+			else
+			{
+				mw_big_direct(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
+			}
+		}
+
+		if(o_direct_write_open == true)
+		{
+			if(ftruncate(ito, from_size_local) == -1)
+			{
+				print_error("ftruncate", __FILE__, __LINE__, cpdd);
+			}
 		}
 	}
 	else if((from_size_local == 0) && (to_open == true))
@@ -388,7 +425,7 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 	}
 
 /*
- * fdatasyncするようにしたら、30倍遅くなった。
+ * ext4でfdatasyncするようにしたら、30倍遅くなった。
 
 	if((oparg->CHECK != NOT) && (fallocate_support == true))
 	{
@@ -439,7 +476,7 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 
 	write_now = NULL;
 
-	if((from_open == true) && (to_open == true) && (from_size_local == total_write_local))
+	if((from_open == true) && (to_open == true) && (from_size_local <= total_write_local))
 	{
 		cpinfo->write = true;
 	}
@@ -559,6 +596,109 @@ inline static void mw_small(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_
 	}
 }
 
+/*******************************************************************************
+*******************************************************************************/
+inline static void mw_small_direct(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
+{
+	off_t read_size;
+	off_t write_size;
+
+	for(;;)
+	{
+		off_t read_remainder = from_size_local - total_read_local;
+
+		if(read_remainder <= 0)
+		{
+			break;
+		}
+
+		off_t surplus = read_remainder % psize;
+
+		if(surplus == 0)
+		{
+			read_size = read_remainder;
+			write_size = read_size;
+		}
+		else if(surplus == read_remainder)
+		{
+			read_size = read_remainder;
+			write_size = psize;
+		}
+		else
+		{
+			read_size = read_remainder - surplus;
+			write_size = read_size;
+		}
+
+		errno = 0;
+		buf[0] = mmap(NULL, read_size, PROT_READ, MAP_PRIVATE, ifrom, total_read_local);
+
+		if(buf[0] == MAP_FAILED)
+		{
+			print_error("mmap", __FILE__, __LINE__, cpdd);
+			break;
+		}
+		else
+		{
+			total_read_local += read_size;
+
+			errno = 0;
+			if(madvise(buf[0], read_size, MADV_SEQUENTIAL) == -1)
+			{
+				print_error("madvise", __FILE__, __LINE__, cpdd);
+			}
+
+			if(verify == true)
+			{
+				thread_argument->state = state;
+				thread_argument->buffer = buf[0];
+				thread_argument->len = read_size;
+
+				pthread_create(t_verify, NULL, do_md5_append, (void *)thread_argument);
+			}
+
+			for(;;)
+			{
+				errno = 0;
+				if((written_size[0] = pwrite(ito, buf[0], write_size, total_write_local)) == -1)
+				{
+					print_error("write", __FILE__, __LINE__, cpdd);
+					break;
+				}
+
+				if(written_size[0] == write_size)
+				{
+					total_write_local += written_size[0];
+
+					if(lseek(ito, write_size, SEEK_CUR) == -1)
+					{
+						print_error("lseek", __FILE__, __LINE__, cpdd);
+					}
+
+					break;
+				}
+			}
+
+			if(verify == true)
+			{
+				pthread_join(*t_verify, NULL);
+			}
+
+			errno = 0;
+
+			if(munmap(buf[0], read_size) != 0)
+			{
+				print_error("munmap", __FILE__, __LINE__, cpdd);
+			}
+		}
+	}
+
+	if(verify == true)
+	{
+		MD5_FINISH
+	}
+}
+
 /*******************************************************************************
 *******************************************************************************/
 inline static void mw_big(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
@@ -657,91 +797,208 @@ LOOP_END:
 
 /*******************************************************************************
 *******************************************************************************/
-inline static void mw_thread(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
+inline static void mw_big_direct(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
 {
-	cpdd->thread_mode_count++;
+	off_t read_size;
+	off_t write_size;
 
+	for(;;)
 	{
-		VList *b_size_list = oparg->b_size_list;
-		b_size_list = b_size_list->tail;
+		off_t read_remainder = from_size_local - total_read_local;
 
-		int *i = (int *)b_size_list->data;
-		int low = *i;
-
-		b_size_list = b_size_list->prev;
-		int high;
-
-		for(;;)
+		if(read_remainder <= 0)
 		{
-			i = (int *)b_size_list->data;
-			high = *i;
+			break;
+		}
 
-			if(high > from_size_local)
+		if(read_remainder >= buffer_size)
+		{
+			read_size = buffer_size;
+			write_size = buffer_size;
+		}
+		else
+		{
+			off_t surplus = read_remainder % psize;
+			read_size = 0;
+			write_size = 0;
+
+			if(surplus == 0)
 			{
-				buffer_size_thread = low;
-				break;
+				read_size = read_remainder;
+				write_size = read_size;
 			}
-			else if(b_size_list->prev == NULL)
+			else if(surplus == read_remainder)
 			{
-				buffer_size_thread = buffer_size_half;
-				break;
+				read_size = read_remainder;
+				write_size = psize;
 			}
 			else
 			{
-				low = high;
-				b_size_list = b_size_list->prev;
+				read_size = read_remainder - surplus;
+				write_size = read_size;
+			}
+		}
+
+		errno = 0;
+		buf[0] = mmap(NULL, read_size, PROT_READ, MAP_PRIVATE, ifrom, total_read_local);
+
+		if(buf[0] == MAP_FAILED)
+		{
+			print_error("mmap", __FILE__, __LINE__, cpdd);
+			break;
+		}
+		else
+		{
+			total_read_local += read_size;
+
+			errno = 0;
+			if(madvise(buf[0], read_size, MADV_SEQUENTIAL) == -1)
+			{
+				print_error("madvise", __FILE__, __LINE__, cpdd);
+			}
+
+			if(verify == true)
+			{
+				thread_argument->state = state;
+				thread_argument->buffer = buf[0];
+				thread_argument->len = read_size;
+
+				pthread_create(t_verify, NULL, do_md5_append, (void *)thread_argument);
+			}
+
+			for(;;)
+			{
+				errno = 0;
+				if((written_size[0] = pwrite(ito, buf[0], write_size, total_write_local)) == -1)
+				{
+					print_error("write", __FILE__, __LINE__, cpdd);
+					break;
+				}
+
+				if(written_size[0] == write_size)
+				{
+					total_write_local += written_size[0];
+
+					if(lseek(ito, write_size, SEEK_CUR) == -1)
+					{
+						print_error("lseek", __FILE__, __LINE__, cpdd);
+					}
+
+					break;
+				}
+			}
+
+			if(verify == true)
+			{
+				pthread_join(*t_verify, NULL);
+			}
+
+			errno = 0;
+			if(munmap(buf[0], read_size) != 0)
+			{
+				print_error("munmap", __FILE__, __LINE__, cpdd);
 			}
 		}
 	}
 
-	rbuf_num = 0;
-	wbuf_num = 1;
+	if(verify == true)
+	{
+		MD5_FINISH
+	}
+}
+
+/*******************************************************************************
+*******************************************************************************/
+#define GET_BUFFER_SIZE \
+cpdd->thread_mode_count++;\
+\
+{\
+	VList *b_size_list = oparg->b_size_list;\
+	b_size_list = b_size_list->tail;\
+\
+	int *i = (int *)b_size_list->data;\
+	int low = *i;\
+\
+	b_size_list = b_size_list->prev;\
+	int high;\
+\
+	for(;;)\
+	{\
+		i = (int *)b_size_list->data;\
+		high = *i;\
+\
+		if(high > from_size_local)\
+		{\
+			buffer_size_thread = low;\
+			break;\
+		}\
+		else if(b_size_list->prev == NULL)\
+		{\
+			buffer_size_thread = buffer_size_half;\
+			break;\
+		}\
+		else \
+		{\
+			low = high;\
+			b_size_list = b_size_list->prev;\
+		}\
+	}\
+}
+
+/*******************************************************************************
+*******************************************************************************/
+inline static void mw_thread(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
+{
+	GET_BUFFER_SIZE
+
+	ri = 0;
+	wi = 1;
 
 	static pthread_t t_read;
-	pthread_create(&t_read, NULL, read_thread_func, (void *)&rbuf_num);
+	pthread_create(&t_read, NULL, read_thread_func, (void *)&ri);
 	pthread_join(t_read, NULL);
 
-	int num_change;
+	int swap;
 
 	for(;;)
 	{
-		num_change = rbuf_num;
-		rbuf_num = wbuf_num;
-		wbuf_num = num_change;
+		swap = ri;
+		ri = wi;
+		wi = swap;
 
-		pthread_create(&t_read, NULL, read_thread_func, (void *)&rbuf_num);
+		pthread_create(&t_read, NULL, read_thread_func, (void *)&ri);
 
 		if(verify == true)
 		{
 			thread_argument->state = state;
-			thread_argument->buffer = buf[wbuf_num];
-			thread_argument->len = next_write_size[wbuf_num];
+			thread_argument->buffer = buf[wi];
+			thread_argument->len = next_write_size[wi];
 
 			pthread_create(t_verify, NULL, do_md5_append, (void *)thread_argument);
 		}
 
-		if(next_write_size[wbuf_num] > 0)
+		if(next_write_size[wi] > 0)
 		{
-			char *tmp = (char *)buf[wbuf_num];
-			const char * const endp = tmp + next_write_size[wbuf_num];
+			char *tmp = (char *)buf[wi];
+			const char * const endp = tmp + next_write_size[wi];
 
 			while(tmp < endp)
 			{
 				errno = 0;
-				if((written_size[wbuf_num] = write(ito, tmp, endp - tmp)) == -1)
+				if((written_size[wi] = write(ito, tmp, endp - tmp)) == -1)
 				{
 					print_error("write", __FILE__, __LINE__, cpdd);
 					total_write_error++;
 					break;
 				}
 
-				tmp += written_size[wbuf_num];
-				total_write_local += written_size[wbuf_num];
+				tmp += written_size[wi];
+				total_write_local += written_size[wi];
 			}
 		}
 		else
 		{
-			written_size[wbuf_num] = 0;
+			written_size[wi] = 0;
 		}
 
 		if(verify == true)
@@ -749,10 +1006,10 @@ inline static void mw_thread(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5
 			pthread_join(*t_verify, NULL);
 		}
 
-		if(next_write_size[wbuf_num] > 0)
+		if(next_write_size[wi] > 0)
 		{
 			errno = 0;
-			if(munmap(buf[wbuf_num], next_write_size[wbuf_num]) == -1)
+			if(munmap(buf[wi], next_write_size[wi]) == -1)
 			{
 				print_error("munmap", __FILE__, __LINE__, cpdd);
 				total_write_error++;
@@ -761,7 +1018,108 @@ inline static void mw_thread(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5
 
 		pthread_join(t_read, NULL);
 
-		if((from_size_local <= total_write_local) || (written_size[wbuf_num] <= 0))
+		if((from_size_local <= total_write_local) || (written_size[wi] <= 0))
+		{
+			break;
+		}
+	}
+
+	if(verify == true)
+	{
+		MD5_FINISH
+	}
+}
+
+/*******************************************************************************
+*******************************************************************************/
+inline static void mw_thread_direct(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
+{
+	GET_BUFFER_SIZE
+
+	ri = 0;
+	wi = 1;
+
+	static pthread_t t_read;
+	pthread_create(&t_read, NULL, read_thread_func, (void *)&ri);
+	pthread_join(t_read, NULL);
+
+	int swap;
+
+	for(;;)
+	{
+		swap = ri;
+		ri = wi;
+		wi = swap;
+
+		pthread_create(&t_read, NULL, read_thread_func, (void *)&ri);
+
+		if(verify == true)
+		{
+			thread_argument->state = state;
+			thread_argument->buffer = buf[wi];
+			thread_argument->len = next_write_size[wi];
+
+			pthread_create(t_verify, NULL, do_md5_append, (void *)thread_argument);
+		}
+
+		if(next_write_size[wi] > 0)
+		{
+			char *tmp = (char *)buf[wi];
+			off_t wtmp = next_write_size[wi];
+			if(wtmp < psize)
+			{
+				wtmp = psize;
+			}
+			const char * const endp = tmp + wtmp;
+
+			while(tmp < endp)
+			{
+				errno = 0;
+				written_size[wi] = pwrite(ito, tmp, endp - tmp, total_write_local);
+
+				if(written_size[wi] == -1)
+				{
+					print_error("write", __FILE__, __LINE__, cpdd);
+					total_write_error++;
+					break;
+				}
+				else if(written_size[wi] == wtmp)
+				{
+					tmp += written_size[wi];
+					total_write_local += written_size[wi];
+
+					if(lseek(ito, total_write_local + written_size[wi], SEEK_SET) == -1)
+					{
+						print_error("write", __FILE__, __LINE__, cpdd);
+						total_write_error++;
+						break;
+					}
+				}
+			}
+		}
+		else
+		{
+			written_size[wi] = 0;
+		}
+
+		if(verify == true)
+		{
+			pthread_join(*t_verify, NULL);
+		}
+
+		if(next_write_size[wi] > 0)
+		{
+			errno = 0;
+			if(munmap(buf[wi], next_write_size[wi]) == -1)
+			{
+				print_error("munmap", __FILE__, __LINE__, cpdd);
+				total_write_error++;
+			}
+		}
+
+		pthread_join(t_read, NULL);
+
+		if((from_size_local <= total_write_local) || (written_size[wi] <= 0))
 		{
 			break;
 		}
@@ -778,7 +1136,7 @@ inline static void mw_thread(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5
 *******************************************************************************/
 static void * read_thread_func(void *unknown)
 {
-	int i = rbuf_num;
+	int i = ri;
 	unknown++;	/* コンパイル時のwarningを出さないようにするため */
 
 	if(from_size_local <= total_read_local)
@@ -796,6 +1154,7 @@ static void * read_thread_func(void *unknown)
 	else
 	{
 		off_t tmp = read_size % psize;
+
 		if(tmp != read_size)
 		{
 			read_size = read_size - tmp;
