@@ -92,7 +92,11 @@ int main(int argc, char **argv)
 
 	if(oparg->M == MOVE)
 	{
-		if((cpdd->mk_file_count > 0) || (cpdd->mk_link_count > 0) || (cpdd->mk_dir_count > 0))
+		if(
+			((oparg->CHECK == NOT) && ((cpdd->mk_file_count > 0) || (cpdd->mk_link_count > 0) || (cpdd->mk_dir_count > 0)))
+			||
+			((oparg->CHECK != NOT) && ((cpdd->mk_link_count > 0) || (cpdd->mk_dir_count > 0)))
+		)
 		{
 			puts("コピー元の削除を開始します...");
 			fflush(stdout);
