@@ -25,8 +25,9 @@ typedef struct copy_infomation
 	enum copy_mode_flag copy_mode;
 	_Bool write;
 	_Bool skip;
-	_Bool compare;
+	_Bool hash_check;
 	_Bool file_test;
+	char *verify_md5;
 	off_t from_size;
 	off_t to_size;
 } CPInfo;
